@@ -483,7 +483,7 @@ write_data (LINCConnection *cnx, QueuedWrite *qw, gboolean queue_write)
 	g_return_val_if_fail (cnx->status == LINC_CONNECTED,
 			      LINC_IO_FATAL_ERROR);
 
-	while (qw->vecs && qw->vecs->iov_len > 0) {
+	while ((qw->nvecs > 0) && (qw->vecs->iov_len > 0)) {
 		int n;
 
 		d_printf ("write_data %ld bytes to fd %d : %s\n",
@@ -522,11 +522,10 @@ write_data (LINCConnection *cnx, QueuedWrite *qw, gboolean queue_write)
 					 (cnx->options & LINC_CONNECTION_NONBLOCKING))
 					return LINC_IO_QUEUED_DATA;
 
-				else if (errno == EBADF) {
+				else if (errno == EBADF)
 					g_warning ("Serious fd usage error %d", cnx->priv->fd);
-					return LINC_IO_FATAL_ERROR;
-				} else
-					return LINC_IO_FATAL_ERROR; /* Unhandlable error */
+				
+				return LINC_IO_FATAL_ERROR; /* Unhandlable error */
 			}
 
 		} else if (n == 0) /* CHECK: is this really an error condition */
@@ -535,7 +534,7 @@ write_data (LINCConnection *cnx, QueuedWrite *qw, gboolean queue_write)
 		else {
 			glong size = n;
 
-			while (n > qw->vecs->iov_len) {
+			while (n >= qw->vecs->iov_len) {
 				n -= qw->vecs->iov_len;
 				qw->nvecs--;
 				qw->vecs++;
